/*
 *	button.h
 *
 *	Button class definition.
 *
 *	(C) Copyright 1994-1998, Pierre ARNAUD, OPaC bright ideas
 *		CH-1437 SUSCEVAZ, Switzerland. All rights reserved.
 */

/******************************************************************************

  This file is part of the OPaC class library ("OPaC").

  OPaC is distributed with NO WARRANTY OF ANY KIND.  No author or distributor
  accepts any responsibility for the consequences of using it, or for whether
  it serves any particular purpose or works at all, unless he or she says so
  in writing. Refer to the OPaC Free Public License (the "License") for full
  details.

  Every copy of OPaC must include a copy of the License, normally in a plain
  ASCII text file named PUBLIC. The License grants you the right to copy,
  modify and redistribute OPaC, but only under certain conditions described
  in the License. The copyright notice and this notice must be preserved on
  all copies.

******************************************************************************/

#ifndef _OPaC_BUTTON_H_
#define	_OPaC_BUTTON_H_

#ifndef _OPaC_OPEXPORT_H_
#include <OPaC/opexport.h>
#endif

OP_INTERFACE (button.h);

#ifndef _OPaC_CONTROL_H_
#include <OPaC/control.h>
#endif

#ifndef _OPaC_XTWIDGET_H_
#include <OPaC/xtwidget.h>
#endif

/****************************************************************/

/*
 *	Button class. There is a more sophisticated view class,
 *	which is providing the same look : the OPaC Cell.
 */

class OP_PUBLIC OPButton : public OPControl, public OPxTitleWidget
{

	static OPCoord		flat_3D_effect;			//	height & width of 3D effect
	static OPRelCoord	press_3D_offset;		//	horizontal & vertical press offset
	
private:
	OPViewRef			icon;					//	icon at the left of the button
	OPViewRef			contents;				//	associated contents (usually a text tag)
	OPViewRef			shortcut;				//	icon/shortcut in the button, at the right
	OPColorPixel		back_color;				//	background color
	OPAtomRef			back_color_name;		//	background color name
	
protected:
	OPButton ();
	virtual ~ OPButton ();
	
	virtual void StoreInstance (OPStorageIn storage);	/* OVERRIDDEN */
	virtual void RestoreInstance (OPStorageIn storage);	/* OVERRIDDEN */
	
	virtual void UpdateButton ();
	virtual void ChangeGeometry (OPRectIn);				/* OVERRIDDEN */
	virtual void SetPressState (Bool ok, Bool = TRUE);	/* OVERRIDDEN */
	
	virtual OPView* ReturnObjectAsView ();
	
public:
	virtual void Delete ();								/* OVERRIDDEN */
	
	virtual Bool SetTitle (OPStringIn text);
	virtual Bool SetText (OPStringIn text, Card8 mode = 0xFF, Bool relief = TRUE);
	
	virtual Bool SetIcon (OPViewIn icon);
	virtual Bool SetContents (OPViewIn contents);
	virtual Bool SetShortcut (OPViewIn shortcut);
	virtual Bool SetBackColor (OPColorPixel color);
	virtual Bool SetBackColor (OPAtomIn color);
	
	virtual Bool GetIcon (OPViewOut icon) const;
	virtual Bool GetContents (OPViewOut contents) const;
	virtual Bool GetShortcut (OPViewOut shortcut) const;
	virtual Bool GetBackColor (OPColorPixelOut color) const;
	virtual Bool GetBackColor (OPAtomOut color) const;
	
	//	Inherited methods :
	
	virtual void GetMinSize (OPCoordOut dx, OPCoordOut dy) const;		/* OVERRIDDEN */
	virtual void GetMaxSize (OPCoordOut dx, OPCoordOut dy) const;		/* OVERRIDDEN */
	
	virtual void ChildChangedSize (OPViewIn child);						/* OVERRIDDEN */
	virtual void Broadcast (OPAtomIn, OPObjectIn);						/* OVERRIDDEN */
	virtual void Draw (OPGrailPortIn, OPRectIn, OPDrawOp = OP_DRAW_ALL);/* OVERRIDDEN */
	
	OP_DEFINE_CLASS (OPButton, OPControl);
};

/****************************************************************/

#endif /* _OPaC_BUTTON_H_ */

