/*
 *	icon.h
 *
 *	The Icon class implements a simple icon view.
 *
 *	(C) Copyright 1994-1998, Pierre ARNAUD, OPaC bright ideas
 *		CH-1437 SUSCEVAZ, Switzerland. All rights reserved.
 */

/******************************************************************************

  This file is part of the OPaC class library ("OPaC").

  OPaC is distributed with NO WARRANTY OF ANY KIND.  No author or distributor
  accepts any responsibility for the consequences of using it, or for whether
  it serves any particular purpose or works at all, unless he or she says so
  in writing. Refer to the OPaC Free Public License (the "License") for full
  details.

  Every copy of OPaC must include a copy of the License, normally in a plain
  ASCII text file named PUBLIC. The License grants you the right to copy,
  modify and redistribute OPaC, but only under certain conditions described
  in the License. The copyright notice and this notice must be preserved on
  all copies.

******************************************************************************/

#ifndef _OPaC_ICON_H_
#define	_OPaC_ICON_H_

#ifndef _OPaC_OPEXPORT_H_
#include <OPaC/opexport.h>
#endif

OP_INTERFACE (icon.h);

#ifndef _OPaC_CONTROL_H_
#include <OPaC/control.h>
#endif

#ifndef	_OPaC_ICOSTYLE_H_
#include <OPaC/icostyle.h>
#endif

/****************************************************************/

/*
 *	Icon class. An icon is also a kind of button.
 */

class OP_PUBLIC OPIcon : public OPControl
{
	
private:
	OPCoord				dx, dy;					//	icon size
	OPLookIcon			icon;					//	icon in normal state
	OPLookIcon			icon_press;				//	icon in pressed state
	OPLookIcon			icon_hilite;			//	icon in highlighted state
	OPLookIcon			icon_dim;				//	icon in dimmed state
	
protected:
	OPIcon ();
	virtual ~ OPIcon ();
	
	virtual void StoreInstance (OPStorageIn storage);	/* OVERRIDDEN */
	virtual void RestoreInstance (OPStorageIn storage);	/* OVERRIDDEN */
	
public:
	virtual void Delete ();								/* OVERRIDDEN */
	
	virtual void Initialise (OPLookIcon icon);
	virtual void Initialise (OPAtomIn, OPAtomIn, OPAtomIn, OPAtomIn);
	virtual void Initialise (OPLookIcon icon, OPLookIcon press, OPLookIcon hilite, OPLookIcon dim);
	virtual Bool SetDefaultSize ();
	
	virtual OPLookIcon ReturnNormalIcon () const;
	virtual OPLookIcon ReturnPressedIcon () const;
	virtual OPLookIcon ReturnHilitedIcon () const;
	virtual OPLookIcon ReturnDimmedIcon () const;
	
	//	Access to the icons by their names (the names map to a unique OPLookIcon
	//	identifier - the mapping is done by the look manager).
	
	virtual Bool SetNormalIcon (OPAtomIn name);
	virtual Bool SetPressedIcon (OPAtomIn name);
	virtual Bool SetHilitedIcon (OPAtomIn name);
	virtual Bool SetDimmedIcon (OPAtomIn name);
	
	virtual Bool GetNormalIcon (OPAtomOut name) const;
	virtual Bool GetPressedIcon (OPAtomOut name) const;
	virtual Bool GetHilitedIcon (OPAtomOut name) const;
	virtual Bool GetDimmedIcon (OPAtomOut name) const;
	
	//	Inherited methods :
	
	virtual void GetMinSize (OPCoordOut dx, OPCoordOut dy) const;/* OVERRIDDEN */
	virtual void GetMaxSize (OPCoordOut dx, OPCoordOut dy) const;/* OVERRIDDEN */
	
	virtual void Broadcast (OPAtomIn, OPObjectIn);						/* OVERRIDDEN */
	virtual void Draw (OPGrailPortIn, OPRectIn, OPDrawOp = OP_DRAW_ALL);/* OVERRIDDEN */
	
	OP_DEFINE_CLASS (OPIcon, OPControl);
};

/****************************************************************/

#endif /* _OPaC_ICON_H_ */
