/*
 *	multiview.h
 *
 *	The Multiple View is a base class for building tabbed views
 *	and step-by-step wizard views.
 *	visible at a given time.
 *
 *	(C) Copyright 1998, Pierre ARNAUD, OPaC bright ideas
 *		CH-1437 SUSCEVAZ, Switzerland. All rights reserved.
 */

/******************************************************************************

  This file is part of the OPaC class library ("OPaC").

  OPaC is distributed with NO WARRANTY OF ANY KIND.  No author or distributor
  accepts any responsibility for the consequences of using it, or for whether
  it serves any particular purpose or works at all, unless he or she says so
  in writing. Refer to the OPaC Free Public License (the "License") for full
  details.

  Every copy of OPaC must include a copy of the License, normally in a plain
  ASCII text file named PUBLIC. The License grants you the right to copy,
  modify and redistribute OPaC, but only under certain conditions described
  in the License. The copyright notice and this notice must be preserved on
  all copies.

******************************************************************************/

#ifndef _OPaC_MULTIVIEW_H_
#define	_OPaC_MULTIVIEW_H_

#ifndef _OPaC_VIEW_H_
#include <OPaC/view.h>
#endif

/****************************************************************/

/*
 *	Base class for views consisting of multiple different
 *	views, of which only one can be visible at a time.
 */

class OP_PUBLIC OPMultiView : public OPView
{
	
protected:
	OPListRef			views;					//	collection of views
	OPViewRef			active_view;			//	currently active view
	Count				active_index;			//	currently active view index
	
protected:
	OPMultiView ();
	virtual ~ OPMultiView ();
	
	virtual void StoreInstance (OPStorageIn storage);	/* OVERRIDDEN */
	virtual void RestoreInstance (OPStorageIn storage);	/* OVERRIDDEN */
	
	virtual void ChangeGeometry (OPRectIn box);			/* OVERRIDDEN */
	virtual void UpdateGeometry (OPRectIn box);
	virtual Bool MakeViewActive (OPViewIn view, Count index);
	
public:
	virtual void Delete ();								/* OVERRIDDEN */
	
	virtual Bool ActivateView (Count index);
	virtual Bool ActivateView (OPViewIn view);
	virtual Count ReturnActiveView () const;
	virtual Bool GetActiveView (OPViewOut view) const;
	virtual Bool GetNthView (Count nth, OPViewOut view) const;
	
	virtual Bool ActivateNextView ();
	virtual Bool ActivatePrevView ();
	
	virtual Bool SetViewList (OPListIn views);
	virtual Bool GetViewList (OPListOut views) const;
	virtual Count CountViews () const;
	
	virtual void AdjustOverallSize ();
	
	//	Inherited methods :
	
	virtual void GetMinSize (OPCoordOut dx, OPCoordOut dy) const; /* OVERRIDDEN */
	virtual void GetMaxSize (OPCoordOut dx, OPCoordOut dy) const; /* OVERRIDDEN */
	
	OP_DEFINE_CLASS (OPMultiView, OPView);
};

/****************************************************************/

#endif /* _OPaC_MULTIVIEW_H_ */

