/*
 *	texttag.h
 *
 *	The Text Tag class implements a simple, constant, text view.
 *
 *	(C) Copyright 1994-1998, Pierre ARNAUD, OPaC bright ideas
 *		CH-1437 SUSCEVAZ, Switzerland. All rights reserved.
 */

/******************************************************************************

  This file is part of the OPaC class library ("OPaC").

  OPaC is distributed with NO WARRANTY OF ANY KIND.  No author or distributor
  accepts any responsibility for the consequences of using it, or for whether
  it serves any particular purpose or works at all, unless he or she says so
  in writing. Refer to the OPaC Free Public License (the "License") for full
  details.

  Every copy of OPaC must include a copy of the License, normally in a plain
  ASCII text file named PUBLIC. The License grants you the right to copy,
  modify and redistribute OPaC, but only under certain conditions described
  in the License. The copyright notice and this notice must be preserved on
  all copies.

******************************************************************************/

#ifndef _OPaC_TEXTTAG_H_
#define	_OPaC_TEXTTAG_H_

#ifndef _OPaC_OPEXPORT_H_
#include <OPaC/opexport.h>
#endif

OP_INTERFACE (texttag.h);

#ifndef _OPaC_CONTROL_H_
#include <OPaC/control.h>
#endif

#ifndef _OPaC_XTWIDGET_H_
#include <OPaC/xtwidget.h>
#endif

/****************************************************************/

/*
 *	Base Text Tag class. This is used by several, somewhat
 *	different, text display classes.
 */

class OP_PUBLIC OPTextTagBase : public OPControl, public OPxTitleWidget
{

protected:
	OPStringRef			text;				//	text to display
	
protected:
	OPTextTagBase ();
	virtual ~ OPTextTagBase ();
	
	virtual void StoreInstance (OPStorageIn storage);	/* OVERRIDDEN */
	virtual void RestoreInstance (OPStorageIn storage);	/* OVERRIDDEN */
	
	virtual OPView* ReturnObjectAsView ();
	
public:
	virtual void Delete ();								/* OVERRIDDEN */
	
	virtual Bool SetTitle (OPStringIn text);
	
	virtual Bool SetText (OPStringIn text);
	virtual Bool GetText (OPStringOut text) const;
	
	OP_DEFINE_CLASS (OPTextTagBase, OPControl);
};

/****************************************************************/

/*
 *	Text Tag class.
 */

class OP_PUBLIC OPTextTag : public OPTextTagBase
{

protected:
	OPGrailFontRef		font;				//	font to be used
	Card32				mode;				//	text positioning mode
	OPColorPixel		text_color;			//	color of text
	
	Bool				use_auto_color;		//	TRUE => use automatic color
	Bool				use_space;			//	TRUE => has left/right space
	
	OPCoord				left_space;			//	left space (kind of margin)
	OPCoord				right_space;		//	right space (kind of margin)
	
protected:
	OPTextTag ();
	virtual ~ OPTextTag ();
	
	virtual void StoreInstance (OPStorageIn storage);	/* OVERRIDDEN */
	virtual void RestoreInstance (OPStorageIn storage);	/* OVERRIDDEN */
	
public:
	virtual void Delete ();								/* OVERRIDDEN */
	
	//	Text definition methods :
	
	virtual Bool SetFont (OPGrailFontIn font);
	virtual Bool SetMode (Card32 mode);
	virtual Bool SetColor (OPColorPixel pixel);
	virtual Bool SetAutomaticColor (Bool yes = TRUE);
	virtual Bool SetLeftSpace (OPCoord space);
	virtual Bool SetRightSpace (OPCoord space);
	virtual Bool EnableSpace (Bool yes = TRUE);
	
	virtual Bool GetFont (OPGrailFontOut text) const;
	virtual Card32 ReturnMode () const;
	virtual OPColorPixel ReturnColor () const;
	virtual Bool ReturnAutomaticColor () const;
	virtual OPCoord ReturnLeftSpace () const;
	virtual OPCoord ReturnRightSpace () const;
	virtual Bool ReturnSpaceEnabled () const;
	
	//	General usage methods :
	
	virtual void GetMinSize (OPCoordOut dx, OPCoordOut dy) const;/* OVERRIDDEN */
	virtual void Draw (OPGrailPortIn, OPRectIn, OPDrawOp = OP_DRAW_ALL);/* OVERRIDDEN */

	OP_DEFINE_CLASS (OPTextTag, OPTextTagBase);
};

/****************************************************************/

#endif /* _OPaC_TEXTTAG_H_ */
