/*
 *	sd_grailfnt.cxx
 *
 *	CAUTION: This file provides an interface for Win32 only.
 *
 *	GRAphical Interface Layer Font representation. This is machine
 *	dependent !
 *
 *	(C) Copyright 1994-1998, Pierre ARNAUD, OPaC bright ideas
 *		CH-1437 SUSCEVAZ, Switzerland. All rights reserved.
 */

/******************************************************************************

  This file is part of the OPaC class library ("OPaC").

  OPaC is distributed with NO WARRANTY OF ANY KIND.  No author or distributor
  accepts any responsibility for the consequences of using it, or for whether
  it serves any particular purpose or works at all, unless he or she says so
  in writing. Refer to the OPaC Free Public License (the "License") for full
  details.

  Every copy of OPaC must include a copy of the License, normally in a plain
  ASCII text file named PUBLIC. The License grants you the right to copy,
  modify and redistribute OPaC, but only under certain conditions described
  in the License. The copyright notice and this notice must be preserved on
  all copies.

******************************************************************************/

#include "sd_grafix.h"

#include <OPaC/database.h>
#include <OPaC/grail.h>
#include <OPaC/new.h>

/****************************************************************/

OP_IMPLEMENT_CLASS (OPGrailFont);

/****************************************************************/

OPGrailFont::OPGrailFont ()
{
	OP_METHOD (OPGrailFont, 0);
	
	this->font      = 0;
	this->ascender  = 0;
	this->descender = 0;
	this->linegap   = 0;
	this->name      = OPStringNull;
	this->style     = OPStringNull;
	this->size      = 0;
}



OPGrailFont::~OPGrailFont ()
{
	OP_METHOD (~OPGrailFont, 0);

	//	how about freeing the surface
}



void
OPGrailFont::StoreInstance (OPStorageIn storage)
{
	OP_METHOD (StoreInstance, 1);
	OP_ARG (1, OPStorageIn, storage);
	super::StoreInstance (storage);
	
	storage << this->name;
	storage << this->style;
	storage << this->size;
	
	storage << OPStorageSync::mark;
}



void
OPGrailFont::RestoreInstance (OPStorageIn storage)
{
	OP_METHOD (RestoreInstance, 1);
	OP_ARG (1, OPStorageIn, storage);
	super::RestoreInstance (storage);
	
	storage >> this->name;
	storage >> this->style;
	storage >> this->size;
	
	storage >> OPStorageSync::mark;
	
	this->RealizeFont ();
}



Bool
OPGrailFont::Initialise (OPStringIn name, OPStringIn style, OPCoord size)
{
	OP_METHOD (Initialise, 3);
	OP_ARG (1, OPStringIn, name);
	OP_ARG (2, OPStringIn, style);
	OP_ARG (3, OPCoord, size);
	
	if (this->font) {
		return FALSE;
	}
	
	if (this->name.IsValid () == FALSE) {
		this->name = OP_NEW_FROM_ZONE (this->ReturnMemZone (), OPString);
	}
	
	if (this->style.IsValid () == FALSE) {
		this->style = OP_NEW_FROM_ZONE (this->ReturnMemZone (), OPString);
	}
	
	this->name->SetString (name);
	this->style->SetString (style);
	
	this->size = size;
	
	return this->RealizeFont ();
}



Bool
OPGrailFont::RealizeFont ()
{
	OP_METHOD (RealizeFont, 0);
	
	if ( (this->name.IsValid ())
	  && (this->style.IsValid ())
	  && (this->size) ) {
		
		//	Extract the string and put it into the logical font structure. If
		//	the string happens to be too long, just truncate it and try anyway.
		
		LatinChar buffer[200];
		Count      max  = sizeof (buffer);
		Count      len  = this->name->CountCharacters ();
		LatinChar* name = buffer;
		
		if (len > max) len = max;
		
		this->name->GetStringCopy (name, max);
		this->font = NEW GfxFontCache;
		
		if (this->font->OpenFace (buffer, (int)this->size) == FALSE) {
			this->font->OpenFace ("Arial.ttf", (int)this->size);
		}
		
		this->ascender  = this->font->TypoAscender ();
		this->descender = this->font->TypoDescender ();
		this->linegap   = this->font->TypoLineGap ();
		
		return TRUE;
	}
	
	return FALSE;
}



void
OPGrailFont::GetFontName (OPStringOut name) const
{
	OP_METHOD (GetFontName, 1);
	OP_ARG (1, OPStringOut, name);
	
	name = this->name;
}



void
OPGrailFont::GetFontStyle (OPStringOut name) const
{
	OP_METHOD (GetFontStyle, 1);
	OP_ARG (1, OPStringOut, name);
	
	name = this->style;
}



OPCoord
OPGrailFont::ReturnFontSize () const
{
	OP_METHOD (ReturnFontSize, 0);
	return (unsigned int)(this->size);
}



OPRelCoord
OPGrailFont::ReturnAscender () const
{
	OP_METHOD (ReturnAscender, 0);
	
	return this->ascender;
}



OPRelCoord
OPGrailFont::ReturnDescender () const
{
	OP_METHOD (ReturnDescender, 0);
	
	return this->descender;
}



OPCoord
OPGrailFont::ReturnWidth (const LatinChar* text) const
{
	OP_METHOD (ReturnWidth, 1);
	OP_ARG (1, const LatinChar*, text);
	
	if ( (text == 0)
	  || (text[0] == 0) ) {
		return 0;
	}
	
	OPCoord width = 0;
	
	if (this->font) {
		
		int length;
		width = 0;
		
		for (length = 0; text[length]; length++) {
			GfxFontCache::GlyphGeom geom;
			this->font->GetGlyphGeometry (text[length], geom);
			width += geom.x_advance;
		}
	}
	
	return width;
}



OPCoord
OPGrailFont::ReturnWidth (const UnicodeChar* text) const
{
	OP_METHOD (ReturnWidth, 1);
	OP_ARG (1, const UnicodeChar*, text);
	
	if ( (text == 0)
	  || (text[0] == 0) ) {
		return 0;
	}
	
	OPCoord width = 0;
	
	if (this->font) {
		
		int length;
		width = 0;
		
		for (length = 0; text[length]; length++) {
			GfxFontCache::GlyphGeom geom;
			this->font->GetGlyphGeometry (text[length], geom);
			width += geom.x_advance;
		}
	}
	
	return width;
}



OPCoord
OPGrailFont::ReturnWidth (OPStringIn text) const
{
	OP_METHOD (ReturnWidth, 1);
	OP_ARG (1, OPStringIn, text);
	
	if (text.IsValid () == FALSE) {
		return 0;
	}
	
	Count len = text->CountCharacters ();
	
	return this->ReturnWidth (text, 0, len);
}



OPCoord
OPGrailFont::ReturnWidth (OPStringIn text, Count start, Count length) const
{
	OP_METHOD (ReturnWidth, 3);
	OP_ARG (1, OPStringIn, text);
	OP_ARG (2, Count, start);
	OP_ARG (3, Count, length);
	
	if (text.IsValid () == FALSE) {
		return FALSE;
	}
	
	OPCoord width = 0;
	Count   len   = text->CountCharacters ();
	Count   pos   = start + length;
	
	if ( (this->font)
	  && (length != 0)
	  && (start < len) ) {
		
		if (pos > len) length -= (pos - len);
		
		UnicodeChar unicode = 0;
		GfxFontCache::GlyphGeom geom;
		
		while (length--) {
			text->GetCharacter (start++, unicode);
			this->font->GetGlyphGeometry (unicode, geom);
			width += geom.x_advance;
		}
	}
	
	return width;
}


