/*
 *	bitsetspy.h
 *
 *	Spy changes in the bitset edition window and modify and display
 *	the real objects when needed.
 *
 *	(C) Copyright 1994-1998, Pierre ARNAUD, OPaC bright ideas
 *		CH-1437 SUSCEVAZ, Switzerland. All rights reserved.
 */

/******************************************************************************

  This file is part of the OPaC class library ("OPaC").

  OPaC is distributed with NO WARRANTY OF ANY KIND.  No author or distributor
  accepts any responsibility for the consequences of using it, or for whether
  it serves any particular purpose or works at all, unless he or she says so
  in writing. Refer to the OPaC Free Public License (the "License") for full
  details.

  Every copy of OPaC must include a copy of the License, normally in a plain
  ASCII text file named PUBLIC. The License grants you the right to copy,
  modify and redistribute OPaC, but only under certain conditions described
  in the License. The copyright notice and this notice must be preserved on
  all copies.

******************************************************************************/

#ifndef _OPaC_BITSETSPY_H_
#define _OPaC_BITSETSPY_H_

#include <OPaC/app.h>
#include <OPaC/values.h>

APP_EXPORT_CLASS (OPBitsetSpy);

APP_DEFINE_REF (OPBitsetSpy, OPObject);

class OPBitsetSpy : public OPObject
{
	enum RefMode
	{
		REF_NONE	 = 0,
		REF_BY_INDEX = 1,
		REF_BY_NAME  = 2,
		REF_BY_PATH  = 3
	};
	
private:
	OPObjectRef			object;
	RefMode				ref_mode;
	Count				ref_index;
	OPAtomRef			ref_name;
	OPStringRef			ref_path;
	OPValueRef			value;
	Card32				mask;
	Card32				bitset;
	
protected:
	OPBitsetSpy ();
	virtual ~ OPBitsetSpy ();
	
public:
	virtual void SetRefObject (OPObjectIn object, Count index, Card32 mask);
	virtual void SetRefObject (OPObjectIn object, OPAtomIn name, Card32 mask);
	virtual void SetRefObjectByPath (OPObjectIn object, OPStringIn path, Card32 mark);
	
	virtual void AddSwitch (OPSwitchIn button, Card32 value);
	virtual void AddSwitchBit (OPSwitchIn button, Card32 bit);
	
	virtual void OnClick (OPObjectIn src, Card32 id);
	
	APP_DEFINE_CLASS (OPBitsetSpy, OPObject);
};

#endif
